@@ -27,6 +27,43 @@ from qutebrowser.misc import objects
 from qutebrowser.keyinput import modeman
 
 
+class FocusWorkaroundEventFilter(QObject):
+
+    """An event filter working Qt 5.11 keyboard focus issues.
+
+    WORKAROUND for https://bugreports.qt.io/browse/QTBUG-68076
+    """
+
+    def __init__(self, win_id, widget, parent=None):
+        super().__init__(parent)
+        self._win_id = win_id
+        self._widget = widget
+
+    def eventFilter(self, _obj, event):
+        """Act on ChildAdded events."""
+        if event.type() != QEvent.ChildAdded:
+            return False
+
+        pass_modes = [usertypes.KeyMode.command,
+                      usertypes.KeyMode.prompt,
+                      usertypes.KeyMode.yesno]
+
+        if modeman.instance(self._win_id).mode in pass_modes:
+            return False
+
+        tabbed_browser = objreg.get('tabbed-browser', scope='window',
+                                    window=self._win_id)
+        current_index = tabbed_browser.widget.currentIndex()
+        try:
+            widget_index = tabbed_browser.widget.indexOf(self._widget.parent())
+        except RuntimeError:
+            widget_index = -1
+        if current_index == widget_index:
+            QTimer.singleShot(0, self._widget.setFocus)
+
+        return False
+
+
 class ChildEventFilter(QObject):
 
     """An event filter re-adding TabEventFilter on ChildEvent.
@@ -41,40 +78,23 @@ class ChildEventFilter(QObject):
         _widget: The widget expected to send out childEvents.
     """
 
-    def __init__(self, eventfilter, widget, win_id, parent=None):
+    def __init__(self, *, eventfilter, widget=None, parent=None):
         super().__init__(parent)
         self._filter = eventfilter
-        assert widget is not None
         self._widget = widget
-        self._win_id = win_id
 
     def eventFilter(self, obj, event):
         """Act on ChildAdded events."""
         if event.type() == QEvent.ChildAdded:
             child = event.child()
-            log.misc.debug("{} got new child {}, installing filter".format(
-                obj, child))
-            assert obj is self._widget
+            log.misc.debug("{} got new child {}, installing filter"
+                           .format(obj, child))
+
+            # Additional sanity check, but optional
+            if self._widget is not None:
+                assert obj is self._widget
+
             child.installEventFilter(self._filter)
-
-            if qtutils.version_check('5.11', compiled=False, exact=True):
-                # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-68076
-                pass_modes = [usertypes.KeyMode.command,
-                              usertypes.KeyMode.prompt,
-                              usertypes.KeyMode.yesno]
-                if modeman.instance(self._win_id).mode not in pass_modes:
-                    tabbed_browser = objreg.get('tabbed-browser',
-                                                scope='window',
-                                                window=self._win_id)
-                    current_index = tabbed_browser.widget.currentIndex()
-                    try:
-                        widget_index = tabbed_browser.widget.indexOf(
-                            self._widget.parent())
-                    except RuntimeError:
-                        widget_index = -1
-                    if current_index == widget_index:
-                        QTimer.singleShot(0, self._widget.setFocus)
-
         elif event.type() == QEvent.ChildRemoved:
             child = event.child()
             log.misc.debug("{}: removed child {}".format(obj, child))
@@ -101,7 +121,6 @@ class TabEventFilter(QObject):
             QEvent.MouseButtonPress: self._handle_mouse_press,
             QEvent.MouseButtonRelease: self._handle_mouse_release,
             QEvent.Wheel: self._handle_wheel,
-            QEvent.ContextMenu: self._handle_context_menu,
             QEvent.KeyRelease: self._handle_key_release,
         }
         self._ignore_wheel_event = False
@@ -116,7 +135,7 @@ class TabEventFilter(QObject):
         Return:
             True if the event should be filtered, False otherwise.
         """
-        is_rocker_gesture = (config.val.input.rocker_gestures and
+        is_rocker_gesture = (config.val.input.mouse.rocker_gestures and
                              e.buttons() == Qt.LeftButton | Qt.RightButton)
 
         if e.button() in [Qt.XButton1, Qt.XButton2] or is_rocker_gesture:
@@ -195,17 +214,6 @@ class TabEventFilter(QObject):
 
         return False
 
-    def _handle_context_menu(self, _e):
-        """Suppress context menus if rocker gestures are turned on.
-
-        Args:
-            e: The QContextMenuEvent.
-
-        Return:
-            True if the event should be filtered, False otherwise.
-        """
-        return config.val.input.rocker_gestures
-
     def _handle_key_release(self, e):
         """Ignore repeated key release events going to the website.
 
@@ -276,6 +284,11 @@ class TabEventFilter(QObject):
         Return:
             True if the event should be filtered, False otherwise.
         """
+        if (not config.val.input.mouse.back_forward_buttons and
+                e.button() in [Qt.XButton1, Qt.XButton2]):
+            # Back and forward on mice are disabled
+            return
+
         if e.button() in [Qt.XButton1, Qt.LeftButton]:
             # Back button on mice which have it, or rocker gesture
             if self._tab.history.can_go_back():
