@@ -162,13 +162,13 @@ class CompletionView(QTreeView):
         pixel_widths = [(width * perc // 100) for perc in column_widths]
 
         delta = self.verticalScrollBar().sizeHint().width()
-        if pixel_widths[-1] > delta:
-            pixel_widths[-1] -= delta
-        else:
-            pixel_widths[-2] -= delta
+        for i, width in reversed(list(enumerate(pixel_widths))):
+            if width > delta:
+                pixel_widths[i] -= delta
+                break
 
         for i, w in enumerate(pixel_widths):
-            assert w >= 0, i
+            assert w >= 0, (i, w)
             self.setColumnWidth(i, w)
 
     def _next_idx(self, upwards):
@@ -205,6 +205,49 @@ class CompletionView(QTreeView):
 
         raise utils.Unreachable
 
+    def _next_page(self, upwards):
+        """Return the index a page away from the selected index.
+
+        Args:
+            upwards: Get previous item, not next.
+
+        Return:
+            A QModelIndex.
+        """
+        old_idx = self.selectionModel().currentIndex()
+        idx = old_idx
+        model = self.model()
+
+        if not idx.isValid():
+            # No item selected yet
+            return model.last_item() if upwards else model.first_item()
+
+        # Find height of each CompletionView element
+        element_height = self.visualRect(idx).height()
+        page_length = self.height() // element_height
+
+        # Skip one pageful, except leave one old line visible
+        offset = -(page_length - 1) if upwards else page_length - 1
+        idx = model.sibling(old_idx.row() + offset, old_idx.column(), old_idx)
+
+        # Skip category headers
+        while idx.isValid() and not idx.parent().isValid():
+            idx = self.indexAbove(idx) if upwards else self.indexBelow(idx)
+
+        if idx.isValid():
+            return idx
+
+        border_item = model.first_item() if upwards else model.last_item()
+
+        # Wrap around if we were already at the beginning/end
+        if old_idx == border_item:
+            return self._next_idx(upwards)
+
+        # Select the first/last item before wrapping around
+        if upwards:
+            self.scrollTo(border_item.parent())
+        return border_item
+
     def _next_category_idx(self, upwards):
         """Get the index of the previous/next category.
 
@@ -238,14 +281,17 @@ class CompletionView(QTreeView):
 
     @cmdutils.register(instance='completion',
                        modes=[usertypes.KeyMode.command], scope='window')
-    @cmdutils.argument('which', choices=['next', 'prev', 'next-category',
-                                         'prev-category'])
+    @cmdutils.argument('which', choices=['next', 'prev',
+                                         'next-category', 'prev-category',
+                                         'next-page', 'prev-page'])
     @cmdutils.argument('history', flag='H')
     def completion_item_focus(self, which, history=False):
         """Shift the focus of the completion menu to another item.
 
         Args:
-            which: 'next', 'prev', 'next-category', or 'prev-category'.
+            which: 'next', 'prev',
+                   'next-category', 'prev-category',
+                   'next-page', or 'prev-page'.
             history: Navigate through command history if no text was typed.
         """
         if history:
@@ -266,12 +312,14 @@ class CompletionView(QTreeView):
 
         selmodel = self.selectionModel()
         indices = {
-            'next': self._next_idx(upwards=False),
-            'prev': self._next_idx(upwards=True),
-            'next-category': self._next_category_idx(upwards=False),
-            'prev-category': self._next_category_idx(upwards=True),
+            'next': lambda: self._next_idx(upwards=False),
+            'prev': lambda: self._next_idx(upwards=True),
+            'next-category': lambda: self._next_category_idx(upwards=False),
+            'prev-category': lambda: self._next_category_idx(upwards=True),
+            'next-page': lambda: self._next_page(upwards=False),
+            'prev-page': lambda: self._next_page(upwards=True),
         }
-        idx = indices[which]
+        idx = indices[which]()
 
         if not idx.isValid():
             return
@@ -424,4 +472,8 @@ class CompletionView(QTreeView):
             if not index.isValid():
                 raise cmdutils.CommandError("No item selected!")
             text = self.model().data(index)
+
+        if not utils.supports_selection():
+            sel = False
+
         utils.set_clipboard(text, selection=sel)
