@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -21,16 +21,20 @@
 
 import sys
 import code
+import typing
 
 from PyQt5.QtCore import pyqtSignal, pyqtSlot, Qt
 from PyQt5.QtWidgets import QTextEdit, QWidget, QVBoxLayout, QApplication
 from PyQt5.QtGui import QTextCursor
 
-from qutebrowser.config import config
+from qutebrowser.config import stylesheet
 from qutebrowser.misc import cmdhistory, miscwidgets
 from qutebrowser.utils import utils, objreg
 
 
+console_widget = None
+
+
 class ConsoleLineEdit(miscwidgets.CommandLineEdit):
 
     """A QLineEdit which executes entered code and provides a history.
@@ -51,8 +55,6 @@ class ConsoleLineEdit(miscwidgets.CommandLineEdit):
             _namespace: The local namespace of the interpreter.
         """
         super().__init__(parent=parent)
-        self._update_font()
-        config.instance.changed.connect(self._update_font)
         self._history = cmdhistory.History(parent=self)
         self.returnPressed.connect(self.on_return_pressed)
 
@@ -102,11 +104,6 @@ class ConsoleLineEdit(miscwidgets.CommandLineEdit):
         else:
             super().keyPressEvent(e)
 
-    @config.change_filter('fonts.debug_console')
-    def _update_font(self):
-        """Set the correct font."""
-        self.setFont(config.val.fonts.debug_console)
-
 
 class ConsoleTextEdit(QTextEdit):
 
@@ -116,18 +113,11 @@ class ConsoleTextEdit(QTextEdit):
         super().__init__(parent)
         self.setAcceptRichText(False)
         self.setReadOnly(True)
-        config.instance.changed.connect(self._update_font)
-        self._update_font()
         self.setFocusPolicy(Qt.ClickFocus)
 
     def __repr__(self):
         return utils.get_repr(self)
 
-    @config.change_filter('fonts.debug_console')
-    def _update_font(self):
-        """Update font when config changed."""
-        self.setFont(config.val.fonts.debug_console)
-
     def append_text(self, text):
         """Append new text and scroll output to bottom.
 
@@ -153,6 +143,12 @@ class ConsoleWidget(QWidget):
         _interpreter: The InteractiveInterpreter to execute code with.
     """
 
+    STYLESHEET = """
+        ConsoleWidget > ConsoleTextEdit, ConsoleWidget > ConsoleLineEdit {
+            font: {{ conf.fonts.debug_console }};
+        }
+    """
+
     def __init__(self, parent=None):
         super().__init__(parent)
         if not hasattr(sys, 'ps1'):
@@ -169,7 +165,7 @@ class ConsoleWidget(QWidget):
             'objreg': objreg,
         }
         self._more = False
-        self._buffer = []
+        self._buffer = []  # type: typing.MutableSequence[str]
         self._lineedit = ConsoleLineEdit(namespace, self)
         self._lineedit.execute.connect(self.push)
         self._output = ConsoleTextEdit()
@@ -178,6 +174,7 @@ class ConsoleWidget(QWidget):
         self._vbox.setSpacing(0)
         self._vbox.addWidget(self._output)
         self._vbox.addWidget(self._lineedit)
+        stylesheet.set_register(self)
         self.setLayout(self._vbox)
         self._lineedit.setFocus()
         self._interpreter = code.InteractiveInterpreter(namespace)
@@ -211,3 +208,9 @@ class ConsoleWidget(QWidget):
     def _curprompt(self):
         """Get the prompt which is visible currently."""
         return sys.ps2 if self._more else sys.ps1
+
+
+def init():
+    """Initialize a global console."""
+    global console_widget
+    console_widget = ConsoleWidget()
