@@ -55,7 +55,12 @@ WEBENGINE_SCHEMES = [
 ]
 
 
-class InvalidUrlError(Exception):
+class Error(Exception):
+
+    """Base class for errors in this module."""
+
+
+class InvalidUrlError(Error):
 
     """Error raised if a function got an invalid URL."""
 
@@ -288,6 +293,11 @@ def is_url(urlstr: str) -> bool:
         # URLs with explicit schemes are always URLs
         log.url.debug("Contains explicit scheme")
         url = True
+    elif (autosearch == 'schemeless' and
+          (not _has_explicit_scheme(qurl) or ' ' in urlstr)):
+        # When autosearch=schemeless, URLs must contain schemes to be valid
+        log.url.debug("No explicit scheme in given URL, treating as non-URL")
+        url = False
     elif qurl_userinput.host() in ['localhost', '127.0.0.1', '::1']:
         log.url.debug("Is localhost.")
         url = True
@@ -619,3 +629,26 @@ def proxy_from_url(url: QUrl) -> typing.Union[QNetworkProxy, pac.PACFetcher]:
     if url.password():
         proxy.setPassword(url.password())
     return proxy
+
+
+def parse_javascript_url(url: QUrl) -> str:
+    """Get JavaScript source from the given URL.
+
+    See https://wiki.whatwg.org/wiki/URL_schemes#javascript:_URLs
+    and https://github.com/whatwg/url/issues/385
+    """
+    ensure_valid(url)
+    if url.scheme() != 'javascript':
+        raise Error("Expected a javascript:... URL")
+    if url.authority():
+        raise Error("URL contains unexpected components: {}"
+                    .format(url.authority()))
+
+    urlstr = url.toString(QUrl.FullyEncoded)  # type: ignore[arg-type]
+    urlstr = urllib.parse.unquote(urlstr)
+
+    code = urlstr[len('javascript:'):]
+    if not code:
+        raise Error("Resulted in empty JavaScript code")
+
+    return code
